-----------------------------------------------------------------------------------------------
-- Client Lua Script for ProfitFlipper
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ProfitFlipper Module Definition
-----------------------------------------------------------------------------------------------
local ProfitFlipper = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ProfitFlipper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ProfitFlipper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {"MarketplaceCommodity"}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ProfitFlipper OnLoad
-----------------------------------------------------------------------------------------------
function ProfitFlipper:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("ProfitFlipper.xml")
	
	self.wndOptions = Apollo.LoadForm(self.xmlDoc, "ProfitFlipperForm", nil, self) 	--nothing yet
	self.wndOptions:Show(false, true)
	
	self.wndCalc = Apollo.LoadForm(self.xmlDoc, "ProfitCalcForm", nil, self)		--profit calculator window
	self.wndCalc:Show(false, true)
	
	self.wndCalc:FindChild("BuyOrderPrice"):Show(true) --gotta set them to show because they arent on be default for some reason
	self.wndCalc:FindChild("BuyOrderFee"):Show(true)
	self.wndCalc:FindChild("BuyOrderTotal"):Show(true)
	self.wndCalc:FindChild("SellOrderPrice"):Show(true)
	self.wndCalc:FindChild("SellOrderFee"):Show(true)
	self.wndCalc:FindChild("SellOrderTotal"):Show(true)
	self.wndCalc:FindChild("ProfitPrice"):Show(true)
	
	Apollo.RegisterSlashCommand("profit", "OnProfitCalc", self)
	
	self:InitializeHooks()													--hooks into the carbine addon
end


-----------------------------------------------------------------------------------------------
-- ProfitFlipper Functions
-----------------------------------------------------------------------------------------------
function ProfitFlipper:InitializeHooks()
	local MarketplaceCommodity = Apollo.GetAddon("MarketplaceCommodity") 	--only need it to be temporary
	
	local fnOldHeaderBtnToggle = MarketplaceCommodity.OnHeaderBtnToggle						--copy the old function..
    MarketplaceCommodity.OnHeaderBtnToggle = function(tMarketplaceCommodity) 				--put in a new one..
		marketplaceWindow  = tMarketplaceCommodity.wndMain
        fnOldHeaderBtnToggle(tMarketplaceCommodity)											--that uses the old one, so as not to undo it

        local children = marketplaceWindow:FindChild("MainScrollContainer"):GetChildren()
		local bBuyOrderChecked = marketplaceWindow:FindChild("HeaderBuyOrderBtn"):IsChecked()
		if bBuyOrderChecked or marketplaceWindow:FindChild("HeaderSellOrderBtn"):IsChecked() then	--make sure we are on the right tab..
	        for i, child in ipairs(children) do														--and for all its list item children..
	            if child:GetText() == "" then
	                local wndProfit = Apollo.LoadForm(self.xmlDoc, "PotentialProfit", child, self)	--put in our profit thingy as a child of the list item
	            end																					--data not put in yet
	        end
		end
		
		local btnHide = marketplaceWindow:FindChild("HideUnprofitableButton")		--figure out if the button is already there
		if btnHide == nil and bBuyOrderChecked then									--if it isn't then make it
			Apollo.LoadForm(self.xmlDoc, "HideUnprofitableButton", tMarketplaceCommodity.wndMain:FindChild("MetalHeader"), MarketplaceCommodity)
		elseif btnHide ~= nil then													--if it is then have it shown or not
			btnHide:Show(bBuyOrderChecked)
			btnHide:SetCheck(false)
		end
	end
	
	local fnOldOnCommodityInfoResults = MarketplaceCommodity.OnCommodityInfoResults			--replacing another function for putting in the data
	MarketplaceCommodity.OnCommodityInfoResults = function(tMarketplaceCommodity, nItemId, tStats, tOrders)
		marketplaceWindow = tMarketplaceCommodity.wndMain
		fnOldOnCommodityInfoResults(tMarketplaceCommodity, nItemId, tStats, tOrders)

		if not marketplaceWindow or not marketplaceWindow:IsValid() then					--gotta check if it's there, just like in original function
			return
		end
		if marketplaceWindow:FindChild("HeaderBuyOrderBtn"):IsChecked() or marketplaceWindow:FindChild("HeaderSellOrderBtn"):IsChecked() then
			local wndMatch = marketplaceWindow:FindChild("MainScrollContainer"):FindChild(nItemId)	--find the one these results are for
			if not wndMatch or not wndMatch:IsValid() then
				return
			end
			
			local wndProfit = wndMatch:FindChild("PotentialProfit")	
		
			local nCompetition = tStats.arBuyOrderPrices[1].monPrice:GetAmount()					--tStats is a bit weird, 
			local nBuyNow = tStats.arSellOrderPrices[1].monPrice:GetAmount()						--index 1 is where the top price is
	
			
			local nPaid = math.floor(nCompetition * 1.02)											--money amounts after fees
			local nReturn = nBuyNow - math.floor(nBuyNow * .12)										--both should correctly be floor
			
			local nProfit = nReturn - nPaid
			local nPercent = nProfit / nPaid * 100													--so if you spend 1g and make 1g30s, thats a 30% profit
			
			if nProfit > 0 and nCompetition > 0 and nBuyNow > 0 then								--if we know there's a profit, put it in
				wndProfit:FindChild("ProfitPercent"):SetText(string.format("%.1f", nPercent).."%")
				wndProfit:FindChild("ProfitPrice"):Show(nProfit)
				wndProfit:FindChild("ProfitPrice"):SetAmount(nProfit)
			elseif nCompetition <= 0 or nBuyNow <= 0 then											--missing data
				wndProfit:FindChild("ProfitPercent"):SetText("Unsure")
				wndProfit:FindChild("ProfitPercent"):SetTextColor(ApolloColor.new("UI_TextHoloBodyCyan"))	--boring color
			else																					--not a profit
				wndProfit:FindChild("ProfitPercent"):SetText("None")
				wndProfit:FindChild("ProfitPercent"):SetTextColor(ApolloColor.new("UI_TextHoloBodyCyan"))
			end
			
			if not marketplaceWindow:FindChild("HeaderSellOrderBtn"):IsChecked() and nCompetition > 0 then
				local nEfficient = math.floor(25000 / nCompetition)	--efficient amount is where minimum buying fee of 5s is as close to 2% of cost as possible
				if nEfficient < 1 then
					wndProfit:FindChild("EfficiencyAmount"):SetText("Any")	--buying any amount of this item will have 2% fee instead of 5s
				elseif nEfficient > 200 then
					wndProfit:FindChild("EfficiencyAmount"):SetText("200+")	--max amount of 200
				else
					wndProfit:FindChild("EfficiencyAmount"):SetText(tostring(nEfficient))
				end
			else
				wndProfit:FindChild("ProfitEfficiency"):SetText("")	--if no competition price data, efficiency makes no sense to calculate or display
				wndProfit:FindChild("EfficiencyAmount"):SetText("")
			end
		end
	end
	
	MarketplaceCommodity.OnHideUnprofitableChecked = function(tMarketplaceCommodity, wndHandler, wndControl, eMouseButton)
		local children = tMarketplaceCommodity.wndMain:FindChild("MainScrollContainer"):GetChildren()	
	    for i, child in ipairs(children) do
			local price = child:FindChild("ProfitPrice")
	        if price and not price:IsShown() then
				child:Show(false)
			end
		end
		tMarketplaceCommodity.wndMain:FindChild("MainScrollContainer"):ArrangeChildrenVert(0)
	end
	
	MarketplaceCommodity.OnHideUnprofitableUnchecked = function(tMarketplaceCommodity, wndHandler, wndControl, eMouseButton)
		local children = tMarketplaceCommodity.wndMain:FindChild("MainScrollContainer"):GetChildren()	
	    for i, child in ipairs(children) do
	        child:Show(true)
		end
		tMarketplaceCommodity.wndMain:FindChild("MainScrollContainer"):ArrangeChildrenVert(0)
	end
end

function ProfitFlipper:OnProfitCalc() --turn on the calculator by /profit
	self.wndCalc:Invoke()
	
	self.wndCalc:FindChild("BuyOrderPrice"):SetAmount(0)
	self.wndCalc:FindChild("SellOrderPrice"):SetAmount(0)
	self:CalculateFeesAndProfit(1, false)
	
	self.wndCalc:FindChild("ProfitPercent"):SetText("None")
end

function ProfitFlipper:CalculateFeesAndProfit(nQuantity, bQuantityChanged) --main calculator method
	local nBuySingle = self.wndCalc:FindChild("BuyOrderPrice"):GetAmount() 	--block of math
	local nBuy = nBuySingle * nQuantity
	local nSell = self.wndCalc:FindChild("SellOrderPrice"):GetAmount() *nQuantity
	local nBuyFee = math.max(500, math.floor(nBuy * 0.02))
	local nSellFee = math.floor(nSell * .12)
	local nBuyTotal = nBuy + nBuyFee
	local nSellTotal = nSell - nSellFee
	local nProfit = nSellTotal - nBuyTotal
	local nPercent = (nProfit) / nBuyTotal * 100
	
	if bQuantityChanged then	--if we don't check this, then it stays highlighted and bad things happen
		self.wndCalc:FindChild("QuantityAmount"):SetText(nQuantity)
	end
	self.wndCalc:FindChild("BuyOrderFee"):SetAmount(nBuyFee)							--block of setting values
	self.wndCalc:FindChild("SellOrderFee"):SetAmount(nSellFee)
	self.wndCalc:FindChild("BuyOrderTotal"):SetAmount(nBuyTotal)
	self.wndCalc:FindChild("SellOrderTotal"):SetAmount(nSellTotal)
	self.wndCalc:FindChild("ProfitPercent"):SetText(string.format("%.1f", nPercent).."%")
	
	local nEfficient = math.floor(25000 / nBuySingle)
	if nEfficient < 1 then
		self.wndCalc:FindChild("EfficiencyAmount"):SetText("Any")	--buying any amount of this item will have 2% fee instead of 5s
	elseif nEfficient > 200 then
		self.wndCalc:FindChild("EfficiencyAmount"):SetText("200+")	--max amount of 200
	else
		self.wndCalc:FindChild("EfficiencyAmount"):SetText(tostring(nEfficient))
	end
	
	if nProfit >= 0 then
		self.wndCalc:FindChild("ProfitPrice"):SetAmount(nProfit)		--stay white if there's a profit
		self.wndCalc:FindChild("ProfitPrice"):SetTextColor(ApolloColor.new("UI_WindowTextDefault"))
		self.wndCalc:FindChild("ProfitPercent"):SetTextColor(ApolloColor.new("UI_WindowTextDefault"))
	else
		self.wndCalc:FindChild("ProfitPrice"):SetAmount(nProfit * -1)	--turn red if there isn't
		self.wndCalc:FindChild("ProfitPrice"):SetTextColor(ApolloColor.new("UI_WindowTextRed"))
		self.wndCalc:FindChild("ProfitPercent"):SetText("None")
		self.wndCalc:FindChild("ProfitPercent"):SetTextColor(ApolloColor.new("UI_WindowTextRed"))
	end
end

function ProfitFlipper:CalculateFromCX(nQuantity, nBuyOrder, nSellOrder)
	if not self.wndCalc:IsShown() then
		self.wndCalc:Invoke()
	end
	self.wndCalc:FindChild("BuyOrderPrice"):SetAmount(nBuyOrder or 0)
	self.wndCalc:FindChild("SellOrderPrice"):SetAmount(nSellOrder or 0)
	self:CalculateFeesAndProfit(nQuantity or 1, true)
end
-----------------------------------------------------------------------------------------------
-- ProfitFlipperForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function ProfitFlipper:OnCancel()
	self.wndOptions:Close() -- hide the window
end

function ProfitFlipper:OnCancelCalc()
	self.wndCalc:Close() -- hide the window
end

function ProfitFlipper:OnCalcQuantityChanged(wndHandler, wndControl, strText)
	local wndParent = self.wndCalc
	local nQuantity = tonumber(strText)
	if nQuantity then
		if nQuantity > 200 then
			wndParent:FindChild("QuantityAmount"):SetText(200)
		elseif nQuantity < 1 then
			wndParent:FindChild("QuantityAmount"):SetText(1)
		end
	else
		nQuantity = 1
		wndParent:FindChild("QuantityAmount"):SetText(1)
	end
	self:CalculateFeesAndProfit(nQuantity, false)
end

function ProfitFlipper:OnCalcBuyOrderChanged(wndHandler, wndControl)
	local nQuantity = tonumber(self.wndCalc:FindChild("QuantityAmount"):GetText())
	self:CalculateFeesAndProfit(nQuantity, false)
end


function ProfitFlipper:OnCalcSellOrderChanged(wndHandler, wndControl)
	local nQuantity = tonumber(self.wndCalc:FindChild("QuantityAmount"):GetText())
	self:CalculateFeesAndProfit(nQuantity, false)
end

function ProfitFlipper:OnRequestCalculator(wndHandler, wndControl, eMouseButton)
	local wndParent = wndHandler:GetParent():GetParent()
	local nSellOrder = wndParent:FindChild("ListSubtitlePriceRight"):GetAmount()
	local nBuyOrder = wndParent:FindChild("ListInputPrice"):GetAmount()
	local nQuantity = tonumber(wndParent:FindChild("ListInputNumber"):GetText())
	self:CalculateFromCX(nQuantity, nBuyOrder, nSellOrder)
end
-----------------------------------------------------------------------------------------------
-- ProfitFlipper Instance
-----------------------------------------------------------------------------------------------
local ProfitFlipperInst = ProfitFlipper:new()
ProfitFlipperInst:Init()
